import { world, system, ItemStack} from '@minecraft/server';
const allowedBlocks = ["minecraft:oak_log", "minecraft:birch_log", "minecraft:spruce_log", "minecraft:acacia_log", "minecraft:dark_oak_log", "minecraft:jungle_log", "minecraft:mangrove_log", "minecraft:cherry_log"]
const leafBlocks = ["minecraft:oak_leaves", "minecraft:birch_leaves", "minecraft:spruce_leaves", "minecraft:acacia_leaves", "minecraft:dark_oak_leaves", "minecraft:jungle_leaves","minecraft:mangrove_leaves", "minecraft:cherry_leaves"]
const saplings = ["minecraft:oak_sapling", "minecraft:birch_sapling", "minecraft:spruce_sapling", "minecraft:acacia_sapling", "minecraft:dark_oak_sapling", "minecraft:jungle_sapling","minecraft:mangrove_sapling", "minecraft:cherry_sapling"]
const dimension = world.getDimension("overworld")
const tickDelay = 5

function objectValueCheck(obj1, obj2) {
    const keys1 = Object.keys(obj1);
    return keys1.every(key => obj1[key] === obj2[key]);
}


function arrayContainsBetter(arr, targetObj) {
    return arr.some(obj => objectValueCheck(obj, targetObj));
}

function checkIfWood(index, x, y, z, direction, wood_depth){                            
    if(wood_depth < 6){
        let blockCheck = dimension.getBlock({x: x, y: y + index, z: z});
        if(!allowedBlocks.includes(blockCheck.typeId)){ // If block is not wood!
            return checkIfWood(index + (1 * direction), x, y, z, direction, wood_depth + 1);
        } else {
            return true;
        }
    } 
    return false;
}

function lookForLeaf(x, y, z, depth){ // search up util we find a leaf
    if(depth < 8){
        let blockCheck = dimension.getBlock({x: x, y: y, z: z})
        if(!leafBlocks.includes(blockCheck.typeId)){
            return lookForLeaf(x, y + 1, z, depth + 1)
        } else {
            return {x: x, y: y, z: z};
        }
    } 
    return false; // couldnt find leaf
}
var lookedUpBlocks = [];
var leafArray = [];
var woodArray = [];
function findLeaves(x, y, z, depthMod){
    lookedUpBlocks = [{x: x, y: y, z: z}];
    leafArray = [{x: x, y: y, z: z}]; 
    findLeavesRec(x, y, z, 0 - depthMod);
    return leafArray;
}
function findWood(x, y, z, depthMod){
    lookedUpBlocks = [{x: x, y: y, z: z}];
    woodArray = []
    findWoodRec(x, y, z, 0 - depthMod)
    return woodArray;

}


// first find leaf, check if any wood is attached to all leaves!
function findWoodRec(x, y, z, depth){
    if(depth < 4){
        // search all blocks touching it
        for(let i = -1; i < 2; i++){
            for(let g = -1; g < 2; g++){
                for(let h = -1; h < 2; h++){
                    let newBlockCoords = {x: x + i, y: y + g, z: z + h};
                    if(!arrayContainsBetter(lookedUpBlocks, newBlockCoords)){ // if block has not been checked already
                        lookedUpBlocks.push(newBlockCoords);
                        let blockCheck = dimension.getBlock(newBlockCoords);
                        if(allowedBlocks.includes(blockCheck.typeId)){ // block is not wood
                            woodArray.push(newBlockCoords)
                        } else {
                            findWoodRec(x + i, y + g, z + h, depth + 1)
                        }
                    }
                }
            }
        }
    }
}

function findLeavesRec(x, y, z, depth){
    if(depth < 4){
        // search all blocks touching it
        for(let i = -1; i < 2; i++){
            for(let g = -1; g < 2; g++){
                for(let h = -1; h < 2; h++){
                    let newBlockCoords = {x: x + i, y: y + g, z: z + h};
                    if(!arrayContainsBetter(lookedUpBlocks, newBlockCoords)){ // if block has not been checked already
                        lookedUpBlocks.push(newBlockCoords);
                        let blockCheck = dimension.getBlock(newBlockCoords);
                        if(leafBlocks.includes(blockCheck.typeId)){ // block is leaf
                            leafArray.push(newBlockCoords);   
                            findLeavesRec(x + i, y + g, z + h, depth + 1)  
                        }
                    }
                }
            }
        }
    }
    
}

function destroyLeaf(leafCoordsArray){
    if(leafCoordsArray.length > 1){
        let leaf = leafCoordsArray[Math.floor(Math.random() * leafCoordsArray.length)]
        let leafBlock = dimension.getBlock(leaf)
        let index = leafCoordsArray.indexOf(leaf)
        leafCoordsArray.splice(index, 1)
        if(leafBlock.typeId != "minecraft:air"){
            // loot
            if(Math.random() < .05){
                let saplingIndex = leafBlocks.indexOf(leafBlock.typeId);
                let leafItemStack = new ItemStack(saplings[saplingIndex], 1)
                dimension.spawnItem(leafItemStack, {x: leaf.x, y: leaf.y, z: leaf.z})
            }
            if(Math.random() < .005){
                let appleItemStack = new ItemStack("minecraft:apple", 1)
                dimension.spawnItem(appleItemStack, {x: leaf.x, y: leaf.y, z: leaf.z})
            }
            if(Math.random() < .02){
                let stickItemStack = new ItemStack("minecraft:stick", 1)
                dimension.spawnItem(stickItemStack, {x: leaf.x, y: leaf.y, z: leaf.z})
            }
            leafBlock.setType("minecraft:air")
        }
        system.runTimeout(() => {
            destroyLeaf(leafCoordsArray)
        }, tickDelay)
    }
}

world.beforeEvents.playerBreakBlock.subscribe(event => {
    const block = event.block;
    if(allowedBlocks.includes(block.typeId)){ // if block is wood
        let checkVert = checkIfWood(1, block.x, block.y, block.z, 1, 0)
        if(!checkVert){
            let woodDepthMod = 0
            if(block.typeId == "minecraft:spruce_wood"){
                woodDepthMod = -1
            }
            let woodCount = findWood(block.x, block.y, block.z, woodDepthMod).length
            if(woodCount == 0){ // couldnt find any wood 
                try{ // looking for leaf can fail
                    let depthMod = 0
                    let leafCheckCoords = lookForLeaf(block.x + 1, block.y - 3, block.z, 0);
                    let leaf = dimension.getBlock(leafCheckCoords)
                    if(leaf.typeId == "minecraft:jungle_leaves"){
                        depthMod = 1 // if jungle tree, then add more depth (big tree)
                    }
                    else if(leaf.typeId == "minecraft:spruce_leaves"){
                        depthMod = 1
                    }
                    let leafCoords = findLeaves(leafCheckCoords.x, leafCheckCoords.y, leafCheckCoords.z, depthMod);
                    system.runTimeout(() => {
                        destroyLeaf(leafCoords)
                    }, 5)
                    return;
                }
                catch (e) {
                    console.warn("Probably could not find leaf")
                    console.warn(e)
                }
            }
            return;
        }
        return;
    }
    return;
});